% Some bookkeeping ... 
clear all;
close all;
clc;
set(0,'defaulttextInterpreter','latex'); 

addpath('Helpers'); % this should refer to the folder 'Helpers' 

rng(6);             % set seed 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% This code replicates the numerical studies in Mesters & Zwiernik (2023) 
%
% There are 4 sections: 
%
% 1: simulation study for common variance components model   
%  
% 2: large scale experiments for common variance components model 
%
% 3: simulation study for multiple scaled elliptical components model 
%
% 4: large scale experiments for multiple scaled elliptical components model  
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% SECTION 1: SIMULATION STUDY COMMON VARIANCE MODEL

% We simulate data from the common variance model 
%
% Y = A^-1 \eps, eps = \tau \eta , where \tau \sim Gamma(1,1) and \eta has independent components that are draws from different distributions
% 
% For each draw we estimate A using non-independent component minimum distance methods and TICA, as well as various ICA methods 

vE            = 1:1:10;                  % indicator eta distribution (cf table 1 in paper for details)
iS            = 1000;                    % # simulations 
iM            = 11;                      % # estimation methods

mResultsAmari = zeros(iM,size(vE,2));    % storage average Amari errors 
mResultsFrobi = zeros(iM,size(vE,2));    % storage average Frobenius errors  

mResAllA      = zeros(iS,iM,size(vE,2)); % store all Amari errors 
mResAllF      = zeros(iS,iM,size(vE,2)); % store all Frobenius errors  

for e=1:size(vE,2)  
    
    mResA     = zeros(iS,iM);             
    mResF     = zeros(iS,iM);
    
    for s=1:iS
        
        % dimensions 
        iD                = 2;
        iL                = iD * (iD-1) / 2; 
        iT                = 200;

        % true parameters
        vA0               = normrnd(0,1,[iL,1]);
        mR0               = fAmatrix(vA0, iD);     % get rotation matrix via Cayley transform 
        mSigma            = eye(iD);
        mLigma            = chol(mSigma,'lower');
        mA0               = mR0' * mLigma;           
       
        % generate data  
        mEta              = zeros(iD,iT);
        for d=1:iD 
            mEta(d,:)     =  fError(e,1,iT); 
        end
        mEta              = mEta * sqrt(1/2);                        
        vTau              = gamrnd(1,1, [1,iT]); 
        vTau              = vTau ./ std(vTau);                   
        mEps              = vTau .* mEta;                                  
        mY                = mA0 \ mEps;  

        % nICA estimation 
        [mAhat, mAhatEff] = fGetTensorMin(3, 1, mY, 0, 20,  [vech(zeros(iD,iD)) ; vA0]);   
        dAmariDiag        = amari_distance(mA0,mAhat); 
        dFrobiDiag        = frobenius_distance(mA0,mAhat); 
        dAmariDiagEff     = amari_distance(mA0,mAhatEff);
        dFrobiDiagEff     = frobenius_distance(mA0,mAhatEff);  
        
        
        [mAhat, mAhatEff] = fGetTensorMin(4, 2, mY, 0, 20, [vech(zeros(iD,iD)) ; vA0]);
        dAmariReflec      = amari_distance(mA0,mAhat); 
        dFrobiReflec      = frobenius_distance(mA0,mAhat); 
        dAmariReflecEff   = amari_distance(mA0,mAhatEff);
        dFrobiReflecEff   = frobenius_distance(mA0,mAhatEff);  
        
        mAhat             = fTICA(mY, vA0);                        % Topographical ICA 
        dAmariTICA        = amari_distance(mA0,mAhat);
        dFrobiTICA        = frobenius_distance(mA0,mAhat);

        
        % ICA methods: 
        [~, mAfastICA, ~, ~] = fastICA(mY,iD);                     
        dAmariFastICA     = amari_distance(mA0,mAfastICA);
        dFrobiFastICA     = frobenius_distance(mA0,mAfastICA);
        
        mAkernelICA       = kernel_ica(mY);
        dAmariKernelICA   = amari_distance(mA0,mAkernelICA);
        dFrobiKernelICA   = frobenius_distance(mA0,mAkernelICA);
       
        mAjadeICA         = jadeR(mY,iD);
        dAmariJadeICA     = amari_distance(mA0,mAjadeICA);
        dFrobiJadeICA     = frobenius_distance(mA0,mAjadeICA);
        
        mAefficientICA    = fEfficientICA(mY, mAjadeICA, log(log(iT)), 0.01, 20);
        dAmariEfficientICA= amari_distance(mA0,mAefficientICA);
        dFrobiEfficientICA= frobenius_distance(mA0,mAefficientICA);
        
        
        % ICA methods from R 
        csvwrite('mY.csv',mY);                                    % write to csv 
        !R CMD BATCH /home/geert/Dropbox/ICA_Piotr/beyond_independence_in_ICA/Simulations/Revision/Replication/Helpers/ICAinR.R 
        
        mAhatProDens      = csvread('outProDens.csv',1,1);
        dAmariProDens     = amari_distance(mA0,mAhatProDens);
        dFrobiProDens     = frobenius_distance(mA0,mAhatProDens);
        
        mAhatNPICA        = csvread('outNPICA.csv',1,1);
        dAmariNP_ICA      = amari_distance(mA0,mAhatNPICA);
        dFrobiNP_ICA      = frobenius_distance(mA0,mAhatNPICA);        
        

        mResA(s,:)         = [dAmariDiag, dAmariDiagEff , dAmariReflec, dAmariReflecEff, dAmariTICA, dAmariFastICA, dAmariKernelICA, dAmariJadeICA, dAmariEfficientICA , dAmariProDens, dAmariNP_ICA]; 
        mResF(s,:)         = [dFrobiDiag, dFrobiDiagEff , dFrobiReflec, dFrobiReflecEff, dFrobiTICA, dFrobiFastICA, dFrobiKernelICA, dFrobiJadeICA, dFrobiEfficientICA , dFrobiProDens, dFrobiNP_ICA]; 
    end
    
    mResultsAmari(:,e) = mean(mResA,1)';     % main results vector 
    mResultsFrobi(:,e) = mean(mResF,1)';     % main results vector 
    
    mResAllA(:,:,e)    = mResA; 
    mResAllF(:,:,e)    = mResF; 
end 




%% SECTION 2: LARGE SCALE EXPERIMENTS COMMON VARIANCE MODEL

% set up case of interest. 
iD       = 4;                % dimension 
iL       = iD * (iD-1) / 2; 
iT       = 200;              % sample size 
e        = 1;                % dens eta 
iS       = 100;              % # simulations 
iM       = 6;                % # estimation methods
mResA    = zeros(iS,iM);     % storage amari 
mResF    = zeros(iS,iM);     % storage frobenius 

for s=1:iS     
    % true parameters
    vA0               = normrnd(0,1,[iL,1]);
    mR0               = fAmatrix(vA0, iD);     % rotation matrix via Cayley transform 
    mSigma            = eye(iD);
    mLigma            = chol(mSigma,'lower');
    mA0               = mR0' * mLigma; 

    % generate data  
    mEta              = zeros(iD,iT);
    for d=1:iD 
        mEta(d,:)     =  fError(e,1,iT); 
    end
    mEta              = mEta * sqrt(1/2);                        
    vTau              = gamrnd(1,1, [1,iT]); 
    vTau              = vTau ./ std(vTau);                   
    mEps              = vTau .* mEta;                                  
    mY                = mA0 \ mEps;  

    % nICA estimation 
    [mAhat, mAhatEff] = fGetTensorMin(3, 1, mY, 0, 20,  [vech(zeros(iD,iD)) ; vA0]);   
    dAmariDiag        = amari_distance(mA0,mAhat); 
    dFrobiDiag        = frobenius_distance(mA0,mAhat); 
    dAmariDiagEff     = amari_distance(mA0,mAhatEff);
    dFrobiDiagEff     = frobenius_distance(mA0,mAhatEff);  


    [mAhat, mAhatEff] = fGetTensorMin(4, 2, mY, 0, 20,  [vech(zeros(iD,iD)) ; vA0]);
    dAmariReflec      = amari_distance(mA0,mAhat); 
    dFrobiReflec      = frobenius_distance(mA0,mAhat); 
    dAmariReflecEff   = amari_distance(mA0,mAhatEff);
    dFrobiReflecEff   = frobenius_distance(mA0,mAhatEff);  

    mAhat             = fTICA(mY, vA0);                        % Topographical ICA 
    dAmariTICA        = amari_distance(mA0,mAhat);
    dFrobiTICA        = frobenius_distance(mA0,mAhat);


    % ICA methods: 
    [~, mAfastICA, ~, ~] = fastICA(mY,iD);                     
    dAmariFastICA     = amari_distance(mA0,mAfastICA);
    dFrobiFastICA     = frobenius_distance(mA0,mAfastICA);

    mResA(s,:)         = [dAmariDiag, dAmariDiagEff , dAmariReflec, dAmariReflecEff, dAmariTICA, dAmariFastICA]; 
    mResF(s,:)         = [dFrobiDiag, dFrobiDiagEff , dFrobiReflec, dFrobiReflecEff, dFrobiTICA, dFrobiFastICA]; 
end


% make box plot figure 
figureA = figure, 
boxplot(mResA, 'symbol', '');
set(gca,'XTickLabel',{' '})
xaxisproperties= get(gca, 'XAxis');
xaxisproperties.TickLabelInterpreter = 'latex'; % latex for x-axis
title('$d = 4$, $n=200$, $\eta_i \sim N$');
xticklabels({'$\mu_3^{d,I}$', '$\mu_3^{d,\widehat{\Sigma}_n^{-1}}$' ,'$\mu_4^{r,I}$', '$\mu_4^{r,\widehat{\Sigma}_n^{-1}}$' , 'TICA', 'Fast' })
ylabel('Amari error'); 
ylim([0,1]);

figureB = figure, 
boxplot(mResF, 'symbol', '');
set(gca,'XTickLabel',{' '})
xaxisproperties= get(gca, 'XAxis');
xaxisproperties.TickLabelInterpreter = 'latex'; % latex for x-axis
title('$d = 4$, $n=200$, $\eta_i \sim N$');
xticklabels({'$\mu_3^{d,I}$', '$\mu_3^{d,\widehat{\Sigma}_n^{-1}}$' ,'$\mu_4^{r,I}$', '$\mu_4^{r,\widehat{\Sigma}_n^{-1}}$' , 'TICA', 'Fast' })
ylabel('Frobenius error'); 

% uncomment to print figures 
% exportgraphics(figureA,'Figures/Figure_commonvariance_d=4_n=200_eta=N_amari.pdf','ContentType','vector','BackgroundColor','none');
% exportgraphics(figureB,'Figures/Figure_commonvariance_d=4_n=200_eta=N_frobi.pdf','ContentType','vector','BackgroundColor','none');


%% SECTION 3: SIMULATION STUDY MULTIPLE SCALE ELLIPTICAL 

% We simulate data from the multiple scaled elliptical 
%
% Y = A^-1 \eps, eps = \tau o \eta , where \tau = J e and \eta is uniform on the unit sphere 
% J is matrix of ones and e has independent components that are simulated from different densities
% 
% For each draw we estimate A using non-independent component minimum distance methods and TICA, as well as various ICA methods 

vE       = 1:1:10;               % indicator eta distribution (cf table 1 for details)
iS       = 1000;                 % # simulations 
iM       = 11;                    % # estimation methods

mResultsAmari = zeros(iM,size(vE,2));  % storage average Amari errors 
mResultsFrobi = zeros(iM,size(vE,2));   % storage average Frobenius errors  

mResAllA      = zeros(iS,iM,size(vE,2));% store all Amari errors 
mResAllF      = zeros(iS,iM,size(vE,2));% store all Frobenius errors  


for e=1:size(vE,2) 
    
    mResA     = zeros(iS,iM);
    mResF     = zeros(iS,iM);
    
    for s=1:iS
        
        % dimensions 
        iD                = 2;
        iL                = iD * (iD-1) / 2; 
        iT                = 200;

        % true parameters
        vA0               = normrnd(0,1,[iL,1]);
        mR0               = fAmatrix(vA0, iD);     % rotation matrix via Cayley transform 
        mSigma            = eye(iD);
        mLigma            = chol(mSigma,'lower');
        mA0               = mR0' * mLigma; 

        % generate data  
        mEta              = [ fError(1,1,iT) ;  fError(1,1,iT)   ] ;                        
        mEta              = mEta ./ sqrt(sum(mEta.^2 ,1));                                  
        mTau              = [[1 , 1] ; [1 , 1]] * [ fError(e,1,iT) ;  fError(e,1,iT)   ] ;  
        mEps              = mTau .* mEta; 
        mY                = mA0 \ mEps;  

       
                % nICA estimation 
        [mAhat, mAhatEff] = fGetTensorMin(3, 1, mY, 0, 20,  [vech(zeros(iD,iD)) ; vA0]);   
        dAmariDiag        = amari_distance(mA0,mAhat); 
        dFrobiDiag        = frobenius_distance(mA0,mAhat); 
        dAmariDiagEff     = amari_distance(mA0,mAhatEff);
        dFrobiDiagEff     = frobenius_distance(mA0,mAhatEff);  
        
        
        [mAhat, mAhatEff] = fGetTensorMin(4, 2, mY, 0, 20, [vech(zeros(iD,iD)) ; vA0]);
        dAmariReflec      = amari_distance(mA0,mAhat); 
        dFrobiReflec      = frobenius_distance(mA0,mAhat); 
        dAmariReflecEff   = amari_distance(mA0,mAhatEff);
        dFrobiReflecEff   = frobenius_distance(mA0,mAhatEff);  
        
        mAhat             = fTICA(mY, vA0);                        % Topographical ICA 
        dAmariTICA        = amari_distance(mA0,mAhat);
        dFrobiTICA        = frobenius_distance(mA0,mAhat);

        
        % ICA methods: 
        [~, mAfastICA, ~, ~] = fastICA(mY,iD);                     
        dAmariFastICA     = amari_distance(mA0,mAfastICA);
        dFrobiFastICA     = frobenius_distance(mA0,mAfastICA);
        
        mAkernelICA       = kernel_ica(mY);
        dAmariKernelICA   = amari_distance(mA0,mAkernelICA);
        dFrobiKernelICA   = frobenius_distance(mA0,mAkernelICA);
       
        mAjadeICA         = jadeR(mY,iD);
        dAmariJadeICA     = amari_distance(mA0,mAjadeICA);
        dFrobiJadeICA     = frobenius_distance(mA0,mAjadeICA);
        
        mAefficientICA    = fEfficientICA(mY, mAjadeICA, log(log(iT)), 0.01, 20);
        dAmariEfficientICA= amari_distance(mA0,mAefficientICA);
        dFrobiEfficientICA= frobenius_distance(mA0,mAefficientICA);
        
        
        % ICA methods from R 
        csvwrite('mY.csv',mY);                                    % write to csv 
        !R CMD BATCH /home/geert/Dropbox/ICA_Piotr/beyond_independence_in_ICA/Simulations/Revision/Replication/Helpers/ICAinR.R 
        
        mAhatProDens      = csvread('outProDens.csv',1,1);
        dAmariProDens     = amari_distance(mA0,mAhatProDens);
        dFrobiProDens     = frobenius_distance(mA0,mAhatProDens);
        
        mAhatNPICA        = csvread('outNPICA.csv',1,1);
        dAmariNP_ICA      = amari_distance(mA0,mAhatNPICA);
        dFrobiNP_ICA      = frobenius_distance(mA0,mAhatNPICA);        
        

        mResA(s,:)         = [dAmariDiag, dAmariDiagEff , dAmariReflec, dAmariReflecEff, dAmariTICA, dAmariFastICA, dAmariKernelICA, dAmariJadeICA, dAmariEfficientICA , dAmariProDens, dAmariNP_ICA]; 
        mResF(s,:)         = [dFrobiDiag, dFrobiDiagEff , dFrobiReflec, dFrobiReflecEff, dFrobiTICA, dFrobiFastICA, dFrobiKernelICA, dFrobiJadeICA, dFrobiEfficientICA , dFrobiProDens, dFrobiNP_ICA]; 
    end
    
    mResultsAmari(:,e) = mean(mResA,1)';     % main results vector 
    mResultsFrobi(:,e) = mean(mResF,1)';     % main results vector 
    
    mResAllA(:,:,e)    = mResA; 
    mResAllF(:,:,e)    = mResF; 
    
end 

%% SECTION 4: LARGE SCALE EXPERIMENTS MULTIPLE SCALED ELLIPTICAL MODEL 

% set up case of interest. 
iD       = 4;                % dimension 
iL       = iD * (iD-1) / 2; 
iT       = 1000;             % sample size 
e        = 1;                % dens eta 
iS       = 100;              % # simulations 
iM       = 6;                % # estimation methods
mResA    = zeros(iS,iM);     % storage amari 
mResF    = zeros(iS,iM);     % storage frobenius 

for s=1:iS     
    % true parameters
    vA0               = normrnd(0,1,[iL,1]);
    mR0               = fAmatrix(vA0, iD);     % rotation matrix via Cayley transform 
    mSigma            = eye(iD);
    mLigma            = chol(mSigma,'lower');
    mA0               = mR0' * mLigma; 

    % generate data  
    mEta              = [ fError(1,1,iT) ;  fError(1,1,iT) ;  fError(1,1,iT)   ] ;                        
    mEta              = mEta ./ sqrt(sum(mEta.^2 ,1));          
    tmp               = zeros(iD,iT); 
    for d=1:iD
       tmp(d,:)      = fError(e,1,iT); 
    end
    mTau              = ones(iD,iD) * tmp;  
    mEps              = mTau .* mEta; 
    mY                = mA0 \ mEps;  

    % nICA estimation 
    [mAhat, mAhatEff] = fGetTensorMin(3, 1, mY, 0, 20,  [vech(zeros(iD,iD)) ; vA0]);   
    dAmariDiag        = amari_distance(mA0,mAhat); 
    dFrobiDiag        = frobenius_distance(mA0,mAhat); 
    dAmariDiagEff     = amari_distance(mA0,mAhatEff);
    dFrobiDiagEff     = frobenius_distance(mA0,mAhatEff);  


    [mAhat, mAhatEff] = fGetTensorMin(4, 2, mY, 0, 20, [vech(zeros(iD,iD)) ; vA0]);
    dAmariReflec      = amari_distance(mA0,mAhat); 
    dFrobiReflec      = frobenius_distance(mA0,mAhat); 
    dAmariReflecEff   = amari_distance(mA0,mAhatEff);
    dFrobiReflecEff   = frobenius_distance(mA0,mAhatEff);  

    mAhat             = fTICA(mY, vA0);                        % Topographical ICA 
    dAmariTICA        = amari_distance(mA0,mAhat);
    dFrobiTICA        = frobenius_distance(mA0,mAhat);


    % ICA methods: 
    [~, mAfastICA, ~, ~] = fastICA(mY,iD);                     
    dAmariFastICA     = amari_distance(mA0,mAfastICA);
    dFrobiFastICA     = frobenius_distance(mA0,mAfastICA);

    mResA(s,:)         = [dAmariDiag, dAmariDiagEff , dAmariReflec, dAmariReflecEff, dAmariTICA, dAmariFastICA ]; 
    mResF(s,:)         = [dFrobiDiag, dFrobiDiagEff , dFrobiReflec, dFrobiReflecEff, dFrobiTICA, dFrobiFastICA ]; 
end


% make box plot figure 
figureA = figure, 
boxplot(mResA, 'symbol', '');
set(gca,'XTickLabel',{' '})
xaxisproperties= get(gca, 'XAxis');
xaxisproperties.TickLabelInterpreter = 'latex'; % latex for x-axis
title('$d = 4$, $n=1000$, $\eta_i \sim ACL$');
xticklabels({'$\mu_3^{d,I}$', '$\mu_3^{d,\widehat{\Sigma}_n^{-1}}$' ,'$\mu_4^{r,I}$', '$\mu_4^{r,\widehat{\Sigma}_n^{-1}}$' , 'TICA', 'Fast' })
ylabel('Amari error'); 
ylim([0,1]);

figureB = figure, 
boxplot(mResF, 'symbol', '');
set(gca,'XTickLabel',{' '})
xaxisproperties= get(gca, 'XAxis');
xaxisproperties.TickLabelInterpreter = 'latex'; % latex for x-axis
title('$d = 4$, $n=1000$, $\eta_i \sim ACL$');
xticklabels({'$\mu_3^{d,I}$', '$\mu_3^{d,\widehat{\Sigma}_n^{-1}}$' ,'$\mu_4^{r,I}$', '$\mu_4^{r,\widehat{\Sigma}_n^{-1}}$' , 'TICA', 'Fast' })
ylabel('Frobenius error'); 

% uncomment to print figures 
% exportgraphics(figureA,'Figures/Figure_ellipical_d=5_n=1000_eta=ACL_amari.pdf','ContentType','vector','BackgroundColor','none');
% exportgraphics(figureB,'Figures/Figure_ellipical_d=5_n=1000_eta=ACL_frobi.pdf','ContentType','vector','BackgroundColor','none');


