% Some bookkeeping ... 
clear all;
close all;
clc;
set(0,'defaulttextInterpreter','latex'); 

addpath('Helpers'); % this should refer to the folder 'Helpers' 

rng(1);             % set seed 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% This code provides a short demo of the function fGetTensorMin that implements the minimum distance methods of Mesters & Zwiernik (2023) 
%
% fGetTensorMin computes minimum distance estimates for the mixing matrix A in AY = eps based on higher order tensor restrictions
% the restrictions are either diagonal or reflectionally invariant. Currently tensors of order r=3 and r=4 are supported. 
% 
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Simulate data from a common variance model with Student t(5) errors 
% 
% Y = A^-1 \eps, eps = \tau \eta , where \tau \sim Gamma(1,1) and \eta ~ t(5)  
% 
% A is parametrized as A = R' Sigma^1/2 
% R is rotation matrix via Cayley transform and Sigma^1/2 is lower triangular   

 
% dimensions 
iD                = 3;                    % dimension nICA model   
iL                = iD * (iD-1) / 2;      % # parameters in rotation 
iT                = 1000;                 % sample size 

% true parameters
vA0               = normrnd(0,1,[iL,1]);
mR0               = fAmatrix(vA0, iD);     % get rotation matrix via Cayley transform 
mSigma            = eye(iD);               % Sigma 
mLigma            = chol(mSigma,'lower');  % Sigma^1/2
mA0               = mR0' * mLigma;         % true A matrix     

% generate data  
mEta              = zeros(iD,iT);          
for d=1:iD 
    mEta(d,:)     =  fError(2,1,iT);       % eta ~ t(5)   fError takes as input 1:10 for different distributions and scaling fixed at 1 and sample size
end
mEta              = mEta * sqrt(1/2);      % standardize eta to have variance 1/2                    
vTau              = gamrnd(1,1, [1,iT]);   % tau ~ gamma(1,1) 
vTau              = vTau ./ std(vTau);     % standarize tau to have variance 2               
mEps              = vTau .* mEta;          % eps                         
mY                = mA0 \ mEps;            % observed data series  

% Estimate nICA model 
% input  : 
%          1: tensor order (restricted to 3,4) 
%          2: type: diagonal = 1 or reflect = 2, 
%          3: data: d rows x n columns 
%          4: whitening : 0 joint estimation , 1 separate Sigma and R 
%          5: number of random starting values 
%          6: (optional) initial value : if whithening = 0 => [vech(Sigma^{1/2}) , vA0]   if whithening = 1 => [vA0]  

% output : mAhat = estimate for A based on W=I, mAhatEff = estimate for A based on W=Sigma^-1 using continuous updating estimator 
 

[mAhat, mAhatEff] = fGetTensorMin(4, 2, mY, 0, 10, [vech(zeros(iD,iD)) ; vA0]);  


% compute the amari and minimum distance index measures 
dAmari        = amari_distance(mA0,mAhat);
dFrobi        = frobenius_distance(mA0,mAhat); 
dAmariEff     = amari_distance(mA0,mAhatEff);
dFrobiEff     = frobenius_distance(mA0,mAhatEff);  

 
